# -*- coding: utf-8 -*-
"""
2-d smoothing
"""
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

__author__ = "yuhao"

import ast
import numpy as np
from scipy import ndimage


def smooth(x, window_len=11, window='hanning'):
    """
    Smooth the data using a window with requested size.

    This method is based on the convolution of a scaled window with the signal.
    The signal is prepared by introducing reflected copies of the signal
    (with the window size) in both ends so that transient parts are minimized
    in the begining and end part of the output signal.

    Parameters
    ----------
    x : ndarray
        the input signal
    window_len : scalar
        the dimension of the smoothing window; should be an odd integer.
    window : scalar
        the type of window from 'flat', 'hanning', 'hamming', 'bartlett',
        'blackman' flat window will produce a moving average smoothing.

    Returns
    -------
    y : ndarray
        the smoothed signal

    Examples
    --------
    >>> t=linspace(-2,2,0.1)
    >>> x=sin(t)+randn(len(t))*0.1
    >>> y=smooth(x)

    See Also
    --------
    numpy.hanning, numpy.hamming, numpy.bartlett, numpy.blackman,
    numpy.convolve scipy.signal.lfilter

    TODO: the window parameter could be the window itself if an array
          instead of a string

    Notes
    -----
    length(output) != length(input), to correct this: return
    y[(window_len/2-1):-(window_len/2)] instead of just y.

    """

    if x.ndim != 1:
        raise ValueError("smooth only accepts 1 dimension arrays.")

    if x.size < window_len:
        raise ValueError("Input vector needs to be bigger than window size.")

    if window_len < 3:
        return x

    if window not in ['flat', 'hanning', 'hamming', 'bartlett', 'blackman']:
        raise ValueError("Window is on of 'flat', 'hanning', 'hamming',\
                         'bartlett', 'blackman'")

    s = np.r_[x[window_len-1:0:-1], x, x[-1:-window_len:-1]]

    if window == 'flat':  # moving average
        w = np.ones(window_len, 'd')
    else:
        # w = eval('numpy.' + window + '(window_len)')
        w = ast.literal_eval('numpy.' + window + '(window_len)')

    y = np.convolve(w/w.sum(), s, mode='valid')
    # return y

    # return y[(window_len/2 - 1): -(window_len/2 + 1)]
    return y[(window_len//2-1): -(window_len//2)]


def smooth_2d(m):
    smoothed = ndimage.filters.gaussian_filter(m, 1)
    # m, n = smoothed.shape
    # for i in xrange(m):
    #     for j in xrange(n):
    #         smoothed[i][j] = float("%.2f" % smoothed[i][j])
    smoothed = np.round(smoothed, 2)

    return smoothed

def smooth_trace(trace_data, window=120):
    data = np.array(trace_data)
    mask = np.isfinite(data)
    smoothed = smooth(data[mask], window_len=window//2, window='flat')
    # using half the window length in order to be consistent with opendtect
    data[mask] = smoothed
    return data



# -*- coding: utf-8 -*-
"""
an interface to a hdf5 storage file

Created on Thu May 10 2018
"""
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

__author__ = "yuhao"

import pandas as pd


class WellStorage(object):
    """
    interface to hdf5 file storing well logs

    this class is designed to accept only LasData.data_frame as input data
    """
    def __init__(self, hdf5_file=None):
        self.hdf5_file = hdf5_file

    @property
    def wells(self):
        well_names = None
        with pd.HDFStore(self.hdf5_file) as store:
            well_names = [key[1:] for key in store.keys()]
        return well_names

    def get_well_data(self, well_name):
        try:
            with pd.HDFStore(self.hdf5_file) as store:
                return store[well_name]
        except KeyError:
            raise KeyError("No well named {}".format(well_name))

    def remove_well(self, well_name):
        try:
            with pd.HDFStore(self.hdf5_file) as store:
                return store.remove(well_name)
        except KeyError:
            raise KeyError("No well named {}".format(well_name))

    def add_well(self, well_name, well_data_frame):
        well_name = well_name.lower().replace('-', '_')
        well_data_frame.to_hdf(self.hdf5_file, well_name)

    def update_well(self, well_name, well_data_frame):
        with pd.HDFStore(self.hdf5_file) as store:
            store[well_name.lower().replace('-', '_')] = well_data_frame

    def logs_into_well(self, well_name, logs_data_frame):
        well_name = well_name.lower().replace('-', '_')
        well_df = self.get_well_data(well_name)
        logs_in_wells = well_df.columns.tolist()
        logs_to_add = logs_data_frame.columns.tolist()
        duplicate_columns = list(set(logs_in_wells).intersection(logs_to_add))
        duplicate_columns.remove("Depth(m)")
        if not duplicate_columns:
            new_df = well_df.join(
                logs_data_frame.set_index("Depth(m)"), on="Depth(m)")
            self.remove_well(well_name)
            self.add_well(well_name, new_df)
        else:
            raise ValueError("Duplicate logs: {}".format(duplicate_columns))



# -*- coding: utf-8 -*-
"""
some utilities regarding pressure calculation
"""
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

from builtins import str#, open

__author__ = "yuhao"

import json
from collections import OrderedDict
from pygeopressure.basic.seisegy import SeiSEGY
from . import Path


def create_seis(name, like):
    """
    Parameters
    ----------
    name : str
    like : SeiSEGY
    """
    # create output segy file
    input_path = Path(like.segy_file)
    output_path = input_path.parent / "{}.sgy".format(name)
    return SeiSEGY(str(output_path), like=str(like.segy_file))


def create_seis_info(segy_object, name):
    """
    Parameters
    ----------
    segy_object : SeiSEGY
    name : str
    """
    file_path = Path(segy_object.segy_file).absolute()
    parent_folder = file_path.parent
    dict_info = OrderedDict([
        ("path", str(file_path)),
        ("inDepth", str(segy_object.inDepth)),
        ("Property_Type", str(segy_object.property_type)),
        ("inline_range", [str(segy_object.startInline),
                          str(segy_object.endInline),
                          str(segy_object.stepInline)]),
        ("crline_range", [str(segy_object.startCrline),
                          str(segy_object.endCrline),
                          str(segy_object.stepCrline)]),
        ("z_range", [str(segy_object.startDepth),
                     str(segy_object.endDepth),
                     str(segy_object.stepDepth)])])
    with open(str(parent_folder / "{}.seis".format(name)), 'w') as fl:
        json.dump(dict_info, fl, indent=4)
